/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2016 fortiss GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.spiderchart;

import static org.fortiss.tooling.spiderchart.gc.AbstractGraphicsSupplier.createImage;
import static org.fortiss.tooling.spiderchart.gc.AbstractGraphicsSupplier.getFont;
import static org.fortiss.tooling.spiderchart.gc.Fonts.SERIF;
import static org.fortiss.tooling.spiderchart.listener.ISpiderChartListener.EVENT_CHART_CLICKED;
import static org.fortiss.tooling.spiderchart.listener.ISpiderChartListener.EVENT_POINT_CLICKED;

import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.CopyOnWriteArrayList;

import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.widgets.Display;
import org.fortiss.tooling.spiderchart.gc.AbstractChartImage;
import org.fortiss.tooling.spiderchart.gc.swt.SpiderChartSwtGraphics;
import org.fortiss.tooling.spiderchart.label.SpiderChartLabel;
import org.fortiss.tooling.spiderchart.label.api.IFloatingObject;
import org.fortiss.tooling.spiderchart.listener.ISpiderChartListener;
import org.fortiss.tooling.spiderchart.model.DataSeries;
import org.fortiss.tooling.spiderchart.plotter.SpiderChartLegend;
import org.fortiss.tooling.spiderchart.plotter.SpiderChartPlotter;
import org.fortiss.tooling.spiderchart.plotter.SpiderChartTitle;
import org.fortiss.tooling.spiderchart.sequence.DataSequence;
import org.fortiss.tooling.spiderchart.style.FillStyle;
import org.fortiss.tooling.spiderchart.style.LineStyle;

/**
 * Actual Spider Chart Diagram
 *
 * @author mondal
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating GREEN Hash: 13207B1B51F8F60D2122999D2EDB0603
 */
public final class SpiderChartOld {

	/** the localized string for numbers */
	private static String numberLocale;

	/** Getter for number locale */
	public static String getNumberLocale() {
		return numberLocale;
	}

	/** Setter for number locale */
	public static void setNumberLocale(final String numberLocale) {
		SpiderChartOld.numberLocale = numberLocale;
	}

	/** The flag to be used to enable selection of data points */
	private boolean activateSelection = false;

	/** Axis Margin */
	private double axisMargin = 0.0625D;

	/** Background Canvas Color */
	private int swtBackgroundCanvasColor = SWT.COLOR_GRAY;

	/** Background Styling Theme */
	private FillStyle backStyle = new FillStyle(SWT.COLOR_GRAY);

	/** Temporary Background Image */
	private AbstractChartImage backTmpImage = null;

	/** Border Style */
	private LineStyle border = null;

	/** Margin from the bottom */
	private final double bottomMargin = 0.125D;

	/** Spider Chart Image */
	private AbstractChartImage chartImage = null;

	/** List of action listeners */
	private final List<ISpiderChartListener> chartListeners = new CopyOnWriteArrayList<>();

	/** Current X Coordinate of the cursor */
	private int currentX;

	/** Current Y Coordinate of the cursor */
	private int currentY;

	/** Previous X Coordinate of the cursor */
	private int cursorLastX = 0;

	/** Previous Y Coordinate of the cursor */
	private int cursorLastY = 0;

	/** the rendered spider chart image */
	private AbstractChartImage finalImage = null;

	/** Container for list of Spider Chart Elements such as Labels */
	private final List<IFloatingObject> floatingObjects = new ArrayList<>();

	/** configuration if X-Axis Screen Margin will be set */
	private final boolean fullXAxis = false;

	/** Height of the spider chart */
	private int height = 0;

	/** last height before the zoom */
	private int lastHeight = -1;

	/** last height before the width */
	private int lastWidth = -1;

	/** Chart Left Margin */
	private final double leftMargin = 0.125D;

	/** Spider Chart Legend */
	private SpiderChartLegend legend;

	/** Spider Chart Legend Margin */
	private final double legendMargin = 0.2D;

	/** minimum height */
	private int minimumHeight = 0;

	/** minimum width */
	private int minimumWidth = 0;

	/** microseconds to wait before a chart refresh */
	private long msecs = 2000L;

	/** X-offset for data points */
	private int offsetX = 0;

	/** Y-offset for data points */
	private int offsetY = 0;

	/** */
	private int originalVirtualHeight = -1;

	/** */
	private int originalVirtualWidth = -1;

	/** The plotters to be used for spider chart plotting. */
	private SpiderChartPlotter plotter;

	/** Right Margin */
	private final double rightMargin = 0.125D;

	/** Spider Chart Label */
	private SpiderChartLabel selectedLabel = null;

	/** The data sequence to be used */
	private DataSequence selectedSeq = null;

	/** Data point as selected */
	private int selectedSeqPoint = -1;

	/** Show Tips on the Spider Chart Points */
	private boolean showTips = true;

	/** Check refresh thread status */
	private boolean stopped = false;

	/** Spider Chart Tip Background Color */
	private final Color tipColor = Display.getCurrent().getSystemColor(SWT.COLOR_YELLOW);

	/** Spider Chart Tip Font */
	private final Font tipFont = getFont(SERIF, SWT.NORMAL, 10);

	/** Spider Chart Tip Font Color */
	private final Color tipFontColor = Display.getCurrent().getSystemColor(SWT.COLOR_BLACK);

	/** Spider Chart Title */
	private SpiderChartTitle title;

	/** Top Margin */
	private final double topMargin = 0.125D;

	/** */
	private int virtualHeight = 0;

	/** */
	private int virtualWidth = 0;

	/** Width of the chart */
	private int width = 0;

	/** Constructor */
	public SpiderChartOld(final SpiderChartTitle t, final SpiderChartPlotter p) {
		this.resetChart(t, p);
	}

	/** Registers the selected chart listener */
	public void addChartListener(final ISpiderChartListener cl) {
		this.chartListeners.add(cl);
	}

	/** Adds the selected floating object */
	public void addFloationgObject(final IFloatingObject obj) {
		this.floatingObjects.add(obj);
	}

	/** Adds the plotter */
	public void setPlotter(SpiderChartPlotter p) {
		plotter = p;
	}

	/** Adds the sequence */
	public void addSeq(DataSeries data) {
		plotter.addSeq(data);
	}

	/** Disposes the spider chart */
	public void dispose() {
		if(plotter != null) {
			plotter.dispose();
		}
		if(this.chartImage != null) {
			this.chartImage.dispose();
		}
		if(this.finalImage != null) {
			this.finalImage.dispose();
		}
		if(this.backTmpImage != null) {
			this.backTmpImage.dispose();
		}
	}

	/** Getter for Axis Margin */
	public double getAxisMargin() {
		return this.axisMargin;
	}

	/** Getter for Background Canvas Color */
	public Color getBackgroundCanvasColor() {
		return Display.getCurrent().getSystemColor(swtBackgroundCanvasColor);
	}

	/** Getter for Background Fill Style */
	public FillStyle getBackStyle() {
		return this.backStyle;
	}

	/** Getter for Background Temporary Image */
	public AbstractChartImage getBackTmpImage() {
		return this.backTmpImage;
	}

	/** Getter for Border */
	public LineStyle getBorder() {
		return this.border;
	}

	/** Getter for Bottom Margin */
	public double getBottomMargin() {
		return this.bottomMargin;
	}

	/** Getter for Chart Image */
	public AbstractChartImage getChartImage() {
		return this.chartImage;
	}

	/** Getter for all registered chart listeners */
	public List<ISpiderChartListener> getChartListeners() {
		return this.chartListeners;
	}

	/** Getter for current X coordinate of the cursor */
	public int getCurrentX() {
		return this.currentX;
	}

	/** Getter for current Y coordinate of the cursor */
	public int getCurrentY() {
		return this.currentY;
	}

	/** Getter for last cursor X coordinate of the cursor */
	public int getCursorLastX() {
		return this.cursorLastX;
	}

	/** Getter for last cursor Y coordinate of the cursor */
	public int getCursorLastY() {
		return this.cursorLastY;
	}

	/** Getter for the last image to be shown as a background to the chart */
	public AbstractChartImage getFinalImage() {
		return this.finalImage;
	}

	/** Getter for all floating objects to be used in the chart */
	public List<IFloatingObject> getFloatingObjects() {
		return this.floatingObjects;
	}

	/** Getter for the height */
	public int getHeight() {
		return this.height;
	}

	/** Getter for last height of the chart before zoom */
	public int getLastHeight() {
		return this.lastHeight;
	}

	/** Getter for the last width of the chart before zoom */
	public int getLastWidth() {
		return this.lastWidth;
	}

	/** Getter for left margin */
	public double getLeftMargin() {
		return this.leftMargin;
	}

	/** Getter for the spider chart legend */
	public SpiderChartLegend getLegend() {
		return this.legend;
	}

	/** Getter for legend margin */
	public double getLegendMargin() {
		return this.legendMargin;
	}

	/** Getter for minimum height */
	public int getMinimumHeight() {
		return this.minimumHeight;
	}

	/** Getter for minimum width */
	public int getMinimumWidth() {
		return this.minimumWidth;
	}

	/** Getter for the microseconds to wait before the chart to be refreshed */
	public long getMsecs() {
		return this.msecs;
	}

	/** Getter for X-coordinate offset */
	public int getOffsetX() {
		return this.offsetX;
	}

	/** Getter for Y-coordinate offset */
	public int getOffsetY() {
		return this.offsetY;
	}

	/** Getter for virtual height */
	public int getOriginalVirtualHeight() {
		return this.originalVirtualHeight;
	}

	/** Getter for virtual width */
	public int getOriginalVirtualWidth() {
		return this.originalVirtualWidth;
	}

	/** Returns the plotter. */
	public SpiderChartPlotter getPlotter() {
		return plotter;
	}

	/** Getter for margin from right */
	public double getRightMargin() {
		return this.rightMargin;
	}

	/** Getter for spider chart label */
	public SpiderChartLabel getSelectedLabel() {
		return this.selectedLabel;
	}

	/** Getter for the sequence to be used */
	public DataSequence getSelectedSeq() {
		return this.selectedSeq;
	}

	/** Getter for selected sequence point */
	public int getSelectedSeqPoint() {
		return this.selectedSeqPoint;
	}

	/** Getter for the first plotter to be used */
	public SpiderChartPlotter getSpiderPlotter() {
		// FIXME: wild cast
		return (SpiderChartPlotter)plotter;
	}

	/** Getter for the tip color */
	public Color getTipColor() {
		return this.tipColor;
	}

	/** Getter for tip font */
	public Font getTipFont() {
		return this.tipFont;
	}

	/** Getter for tip font color */
	public Color getTipFontColor() {
		return this.tipFontColor;
	}

	/** Getter for the spider chart title */
	public SpiderChartTitle getTitle() {
		return this.title;
	}

	/** Getter for the margin from top */
	public double getTopMargin() {
		return this.topMargin;
	}

	/** Getter for virtual height */
	public int getVirtualHeight() {
		return this.virtualHeight;
	}

	/** Getter for virtual width */
	public int getVirtualWidth() {
		return this.virtualWidth;
	}

	/** Getter for the width of the chart */
	public int getWidth() {
		return this.width;
	}

	/**
	 * Getter for the configuration to check if it is a active sequence
	 * selection
	 */
	public boolean isActivateSelection() {
		return this.activateSelection;
	}

	/**
	 * Getter for the configuration to check if X-axis screen margin will be set
	 */
	public boolean isFullXAxis() {
		return this.fullXAxis;
	}

	/** Getter for the configuration to check if tips to be shown */
	public boolean isShowTips() {
		return this.showTips;
	}

	/**
	 * Getter to check if the thread responsible to refresh the chart is stopped
	 */
	public boolean isStopped() {
		return this.stopped;
	}

	/** Mouse Click Functionality */
	public void mouseClick() {
		if(this.selectedSeq != null && this.selectedSeqPoint >= 0 || this.selectedLabel != null) {
			this.triggerChartEvent(EVENT_POINT_CLICKED);
			return;
		}
		this.triggerChartEvent(EVENT_CHART_CLICKED);
	}

	/** Mouse Move Functionality */
	public void mouseMoved(final int eX, final int eY) {
		// FIXME: mouse motion feature currently disabled
		// if(plotter == null) {
		// return;
		// }
		//
		// this.currentX = eX;
		// this.currentY = eY;
		//
		// Object previousSelectedObject = this.selectedSeq;
		// final int previousPoint = this.selectedSeqPoint;
		//
		// if(this.selectedSeq == null && this.selectedLabel != null) {
		// previousSelectedObject = this.selectedLabel;
		// }
		// this.selectedSeq = null;
		// this.selectedLabel = null;
		// this.selectedSeqPoint = -1;
		// if(this.activateSelection) {
		// List<DataSequence> sList = plotter.getData();
		// for(DataSequence d : sList) {
		// for(int i = 0; i < d.getHotAreas().size(); i++) {
		// if(((Polygon)d.getHotAreas().get(i)).contains(this.currentX + this.offsetX,
		// this.currentY + this.offsetY)) {
		// boolean triggerEnter = false;
		// if(previousSelectedObject == null) {
		// triggerEnter = true;
		// } else if(previousSelectedObject != d || previousPoint != i) {
		// this.triggerChartEvent(EVENT_LEAVE_POINT);
		// triggerEnter = true;
		// }
		// this.selectedSeq = d;
		// this.selectedSeqPoint = i;
		// if(!triggerEnter) {
		// break;
		// }
		// this.triggerChartEvent(EVENT_ENTER_POINT);
		// break;
		// }
		// }
		// }
		// }
		// if(Math.abs(this.currentX - this.cursorLastX) > 2 ||
		// Math.abs(this.currentY - this.cursorLastY) > 2) {
		// this.cursorLastX = this.currentX;
		// this.cursorLastY = this.currentY;
		// this.triggerChartEvent(EVENT_TIP_UPDATE);
		// }
		// if(previousSelectedObject != null && this.selectedSeq == null && this.selectedLabel ==
		// null) {
		// this.triggerChartEvent(EVENT_LEAVE_POINT);
		// }
	}

	/** Paints the chart */
	public void paint(SpiderChartSwtGraphics graphics) {
		this.floatingObjects.clear();

		if(plotter == null) {
			Color color = Display.getCurrent().getSystemColor(SWT.COLOR_RED);
			graphics.drawText("No plotters have been found", 30, 30, color);
			return;
		}

		resize();

		if(plotter != null) {
			plotter.plotBackground(graphics);
			plotter.drawAxes(graphics);
			plotter.plot(graphics);
			// FIXME: break out
			// return;
		}

		if(title != null) {
			title.draw(graphics);
		}

		if(legend != null) {
			legend.draw(graphics);
			// FIXME: break out
			return;
		}

		// FIXME: untested parts follow
		SpiderChartSwtGraphics gScroll = graphics;
		SpiderChartSwtGraphics gBack = graphics;
		SpiderChartSwtGraphics g = graphics;
		if(this.lastWidth != this.width || this.lastHeight != this.height) {
			this.lastWidth = this.width;
			this.lastHeight = this.height;
		}
		if(this.originalVirtualHeight == -1) {
			this.originalVirtualHeight = this.virtualHeight;
		}
		if(this.originalVirtualWidth == -1) {
			this.originalVirtualWidth = this.virtualWidth;
		}
		if(this.finalImage != null) {
			this.finalImage.dispose();
		}
		this.finalImage = createImage(this.getWidth(), this.getHeight());
		g = this.finalImage.getGraphics();
		gScroll = g;
		gBack = g;
		if(this.backStyle != null) {
			this.backStyle.draw(gBack, getBackgroundCanvasColor(), 0, 0, this.virtualWidth,
					this.virtualHeight);
		}
		// FIXME: scale feature disabled
		// if(plotter.getxScale() != null) {
		// plotter.getxScale().setScreenMax(plotter.getX() + plotter.getWidth());
		// plotter.getxScale().setScreenMaxMargin(
		// (int)(plotter.getxScale().getScreenMax() * (1.0D - this.axisMargin)));
		// if(this.fullXAxis) {
		// plotter.getxScale().setScreenMaxMargin(plotter.getxScale().getScreenMax());
		// }
		// plotter.getxScale().setScreenMin(plotter.getX());
		// }
		// if(plotter.getyScale() != null) {
		// plotter.getyScale().setScreenMax(plotter.getY() + plotter.getHeight());
		// plotter.getyScale().setScreenMaxMargin(
		// (int)(plotter.getyScale().getScreenMax() * (1.0D - this.axisMargin)));
		// plotter.getyScale().setScreenMin(plotter.getY());
		// }
		plotter.plotBackground(gScroll);
		plotter.plot(gScroll);
		if(this.border != null) {
			this.border.drawRect(g, 0, 0, this.getWidth() - 1, this.getHeight() - 1);
		}
		if(this.chartImage != null) {
			final int x1 = plotter.getX();
			final int x2 = plotter.getX() + plotter.getVisibleWidth();
			final int y1 = plotter.getY() - plotter.getDepth();
			final int y2 = plotter.getY() - plotter.getDepth() + plotter.getVisibleHeight();

			g.drawImage(this.chartImage, x1, y1, x2, y2, x1 + this.offsetX, y1 + this.offsetY, x2 +
					this.offsetX, y2 + this.offsetY);
		}
		if(this.chartListeners != null) {
			for(int i = 0; i < this.chartListeners.size(); i++) {
				this.chartListeners.get(i).onPaintUserExit(this, g);
			}
		}

		if(this.finalImage != null) {
			graphics.drawImage(this.finalImage, 0, 0, this.getWidth(), this.getHeight(), 0, 0,
					this.getWidth(), this.getHeight());
		}
		if(gScroll != graphics) {
			gScroll.dispose();
		}
		if(gBack != graphics) {
			gBack.dispose();
		}
		if(g != graphics) {
			g.dispose();
		}
	}

	/**
	 * Places the defined floating object
	 * 
	 * @param obj
	 *            the floating object to place
	 */
	public void placeFloatingObject(final IFloatingObject obj) {
		// To be implemented
	}

	/** Removes all registered listeners */
	public void removeAllChartListener() {
		this.chartListeners.clear();
	}

	/** Removes the specified chart listener */
	public void removeChartListener(final ISpiderChartListener cl) {
		this.chartListeners.remove(cl);
	}

	/** Release the used resources */
	protected void resetChart(final SpiderChartTitle t, final SpiderChartPlotter p) {
		this.legend = null;
		this.title = null;
		this.border = null;
		this.backStyle = null;
		this.selectedSeq = null;
		this.selectedSeqPoint = -1;
		this.floatingObjects.clear();
		plotter = p;
		this.title = t;
	}

	/** Resizes the chart */
	private void resize() {
		final int myHeight = this.getHeight();
		final int myWidth = this.getWidth();
		if(this.virtualWidth < myWidth) {
			this.virtualWidth = myWidth;
		}
		if(this.virtualHeight < myHeight) {
			this.virtualHeight = myHeight;
		}
		plotter.setVisibleWidth((int)(myWidth * (1.0D - (this.legendMargin + this.leftMargin))));
		plotter.setVisibleHeight((int)(myHeight * (1.0D - (this.topMargin + this.bottomMargin))));
		plotter.setX((int)(myWidth * this.leftMargin));
		plotter.setY((int)(myHeight * this.topMargin));
		plotter.setWidth(this.virtualWidth - (myWidth - plotter.getVisibleWidth()));
		plotter.setHeight(this.virtualHeight - (myHeight - plotter.getVisibleHeight()));

		this.title.setX(0);
		this.title.setY(0);
		this.title.setHeight((int)(myHeight * this.topMargin));
		this.title.setWidth(myWidth);

		if(this.legend != null) {
			this.legend.setX((int)(myWidth * (1.0D - this.legendMargin)));
			this.legend.setY((int)(myHeight * this.topMargin));
			this.legend.setWidth((int)(myWidth * this.legendMargin));
			this.legend.setHeight((int)(myHeight * 0.5D));
		}
	}

	/** Setter for the active sequence selection */
	public void setActivateSelection(final boolean activateSelection) {
		this.activateSelection = activateSelection;
	}

	/** Setter for the axis margin */
	public void setAxisMargin(final double axisMargin) {
		this.axisMargin = axisMargin;
	}

	/** Setter for the background canvas color */
	public void setBackgroundCanvasColor(int swtBackgroundCanvasColor) {
		this.swtBackgroundCanvasColor = swtBackgroundCanvasColor;
	}

	/** Setter for the background fill style */
	public void setBackStyle(final FillStyle backStyle) {
		this.backStyle = backStyle;
	}

	/** Setter for background temporary image */
	public void setBackTmpImage(final AbstractChartImage backTmpImage) {
		this.backTmpImage = backTmpImage;
	}

	/** Setter for the border */
	public void setBorder(final LineStyle border) {
		this.border = border;
	}

	/** Setter for the chart image */
	public void setChartImage(final AbstractChartImage chartImage) {
		this.chartImage = chartImage;
	}

	/** Setter for the current X Coordinate of the cursor */
	public void setCurrentX(final int currentX) {
		this.currentX = currentX;
	}

	/** Setter for the current Y Coordinate of the cursor */
	public void setCurrentY(final int currentY) {
		this.currentY = currentY;
	}

	/** Setter for previous X Coordinate of the cursor */
	public void setCursorLastX(final int cursorLastX) {
		this.cursorLastX = cursorLastX;
	}

	/** Setter for previous Y Coordinate of the cursor */
	public void setCursorLastY(final int cursorLastY) {
		this.cursorLastY = cursorLastY;
	}

	/** Setter for final image to be used on the chart */
	public void setFinalImage(final AbstractChartImage finalImage) {
		this.finalImage = finalImage;
	}

	/** Setter for the height of the chart */
	public void setHeight(final int h) {
		if(h > this.minimumHeight) {
			this.height = h;
		}
	}

	/** Setter for the last height before zoom */
	public void setLastHeight(final int lastHeight) {
		this.lastHeight = lastHeight;
	}

	/** Setter for the last width before zoom */
	public void setLastWidth(final int lastWidth) {
		this.lastWidth = lastWidth;
	}

	/** Setter for the legend */
	public void setLegend(final SpiderChartLegend legend) {
		this.legend = legend;
	}

	/** Setter for the minimum height */
	public void setMinimumHeight(final int minimumHeight) {
		this.minimumHeight = minimumHeight;
	}

	/** Setter for the minimum size */
	public void setMinimumSize(final int w, final int h) {
		this.minimumWidth = w;
		this.minimumHeight = h;
	}

	/** Setter for the minimum width */
	public void setMinimumWidth(final int minimumWidth) {
		this.minimumWidth = minimumWidth;
	}

	/** Setter for the miliseconds to wait before the chart gets refreshed */
	public void setMsecs(final long msecs) {
		this.msecs = msecs;
	}

	/** Setter for the offset to use in X-axis */
	public void setOffsetX(final int offsetX) {
		this.offsetX = offsetX;
	}

	/** Setter for the offset to use in Y-axis */
	public void setOffsetY(final int offsetY) {
		this.offsetY = offsetY;
	}

	/** Setter for the virtual height */
	public void setOriginalVirtualHeight(final int originalVirtualHeight) {
		this.originalVirtualHeight = originalVirtualHeight;
	}

	/** Setter for the virtual width */
	public void setOriginalVirtualWidth(final int originalVirtualWidth) {
		this.originalVirtualWidth = originalVirtualWidth;
	}

	/** Setter for the spider chart label */
	public void setSelectedLabel(final SpiderChartLabel selectedLabel) {
		this.selectedLabel = selectedLabel;
	}

	/** Setter for the selected data points sequence */
	public void setSelectedSeq(final DataSequence selectedSeq) {
		this.selectedSeq = selectedSeq;
	}

	/** */
	public void setSelectedSeqPoint(final int selectedSeqPoint) {
		this.selectedSeqPoint = selectedSeqPoint;
	}

	/** Setter for the configuration to show tips */
	public void setShowTips(final boolean showTips) {
		this.showTips = showTips;
	}

	/** Setter to set the size */
	public void setSize(final int w, final int h) {
		this.setWidth(w);
		this.setHeight(h);
	}

	/** Setter for the configuration to mark the stop as stopped */
	public void setStopped(final boolean stopped) {
		this.stopped = stopped;
	}

	/** Setter for the spider chart title */
	public void setTitle(final SpiderChartTitle title) {
		this.title = title;
	}

	/** Setter for the virtual height */
	public void setVirtualHeight(final int virtualHeight) {
		this.virtualHeight = virtualHeight;
	}

	/** Setter for the virtual width */
	public void setVirtualWidth(final int virtualWidth) {
		this.virtualWidth = virtualWidth;
	}

	/** Setter for the width of the chart */
	public void setWidth(final int w) {
		if(w > this.minimumWidth) {
			this.width = w;
		}
	}

	/** Triggers the chart event */
	private void triggerChartEvent(final int event) {
		for(int i = 0; i < this.chartListeners.size(); i++) {
			this.chartListeners.get(i).onChartEvent(this, event);
		}
	}
}
