/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2016 fortiss GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.spiderchart.plotter;

import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontData;
import org.eclipse.swt.widgets.Display;
import org.fortiss.tooling.spiderchart.model.SpiderChart;
import org.fortiss.tooling.spiderchart.style.FillStyle;
import org.fortiss.tooling.spiderchart.style.LineStyle;

/**
 * Base class for elements (title, axes, legend, data plots) of the spider chart.
 *
 * @author mondal
 * @author hoelzl
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating GREEN Hash: 13207B1B51F8F60D2122999D2EDB0603
 */
public abstract class SpiderChartComponentBase {

	/** The spider chart this component belongs to. */
	protected final SpiderChart chart;

	/** The height of the component. */
	protected int height;

	/** The width of the component. */
	protected int width;

	/** The screen x-axis value. */
	protected int x;

	/** The screen y-axis value. */
	protected int y;

	/** The {@link SWT} foreground system color. */
	private int swtForegroundColor;

	/** The {@link SWT} background system color. */
	private int swtBackgroundColor;

	/** The background fill style. */
	private FillStyle backgroundFillStyle;

	/** The line style. */
	private LineStyle lineStyle;

	/** The font data. */
	private FontData fontData = new FontData("Verdana", 10, SWT.NORMAL);

	/** Constructor. */
	public SpiderChartComponentBase(SpiderChart chart) {
		this.chart = chart;
	}

	/** Sets the background color. */
	public final void setSwtBackgroundColor(int swtBackgroundColor) {
		this.swtBackgroundColor = swtBackgroundColor;
	}

	/** Sets the foreground color. */
	public final void setSwtForegroundColor(int swtForegroundColor) {
		this.swtForegroundColor = swtForegroundColor;
	}

	/** Sets the background fill style. */
	public final void setBackgroundFillStyle(FillStyle backgroundFillStyle) {
		this.backgroundFillStyle = backgroundFillStyle;
	}

	/** Sets the line style. */
	public final void setLineStyle(LineStyle lineStyle) {
		this.lineStyle = lineStyle;
	}

	/** Sets font parameters. */
	public final void setFontName(String fontName, int fontSize, int swtFontStyle) {
		fontData.setName(fontName);
		fontData.setHeight(fontSize);
		fontData.setStyle(swtFontStyle);
	}

	/** Sets x. */
	public void setX(int x) {
		this.x = x;
	}

	/** Sets y. */
	public void setY(int y) {
		this.y = y;
	}

	/** Sets width. */
	public void setWidth(int width) {
		this.width = width;
	}

	/** Sets height. */
	public void setHeight(int height) {
		this.height = height;
	}

	/** Returns the {@link Color} of the foreground. NEVER dispose this color object. */
	public final Color getForegroundColor() {
		return Display.getCurrent().getSystemColor(swtForegroundColor);
	}

	/** Returns the {@link Color} of the background. NEVER dispose this color object. */
	public final Color getBackgroundColor() {
		return Display.getCurrent().getSystemColor(swtBackgroundColor);
	}

	/** Returns the background fill style. */
	public final FillStyle getBackgroundFillStyle() {
		return backgroundFillStyle;
	}

	/** Returns the line style. */
	public final LineStyle getLineStyle() {
		return lineStyle;
	}

	/** Creates a new font instance. ALWAYS dispose this afterwards. */
	public Font createFont() {
		return new Font(Display.getCurrent(), fontData);
	}

	/** Returns x. */
	public final int getX() {
		return x;
	}

	/** Returns y. */
	public final int getY() {
		return y;
	}

	/** Returns width. */
	public final int getWidth() {
		return width;
	}

	/** Returns height. */
	public final int getHeight() {
		return height;
	}

	/** Returns the font data. */
	public FontData getFontData() {
		return fontData;
	}
}
