/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2016 fortiss GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.spiderchart.plotter;

import static org.fortiss.tooling.spiderchart.style.LineStyle.NORMAL_LINE;

import java.util.ArrayList;
import java.util.List;

import org.conqat.lib.commons.collections.Pair;
import org.eclipse.swt.graphics.Font;
import org.fortiss.tooling.spiderchart.gc.swt.SpiderChartSwtGraphics;
import org.fortiss.tooling.spiderchart.label.SpiderChartLabel;
import org.fortiss.tooling.spiderchart.model.DataSeries;
import org.fortiss.tooling.spiderchart.model.SpiderChart;
import org.fortiss.tooling.spiderchart.style.LineStyle;

/**
 * Class representing the spiderchart's legend.
 *
 * @author mondal
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating GREEN Hash: 13207B1B51F8F60D2122999D2EDB0603
 */
public final class SpiderChartLegend extends SpiderChartComponentBase {

	/** Constructor. */
	public SpiderChartLegend(SpiderChart chart) {
		super(chart);
	}

	/** The items in the legend. */
	private final List<Pair<DataSeries, LineStyle>> items = new ArrayList<>();

	/** The legend label text. */
	private String labelText = "";

	/** The inner margins. */
	private int margin = 10;

	/** The offset used to position the legend on vertical. */
	private int verticalOffset = 250;
	/** Flag layouting the legend vertically. */
	private boolean verticalLayout = true;

	/** */
	public void addItem(DataSeries data, int swtColor) {
		LineStyle ls = LineStyle.of(1, swtColor, NORMAL_LINE);
		items.add(new Pair<DataSeries, LineStyle>(data, ls));
	}

	/** Draws the provided graphics */
	public void draw(SpiderChartSwtGraphics g) {
		if(labelText != null && labelText.length() > 0) {
			SpiderChartLabel cl = new SpiderChartLabel(labelText, "", false, true);
			cl.paint(g, x, y, width, height);
			return;
		}

		Font ontheflyFont = createFont();
		if(verticalLayout) {
			drawVertical(g, ontheflyFont);
		} else {
			drawHorizontal(g, ontheflyFont);
		}
		ontheflyFont.dispose();
	}

	/** draws the graphics horizontally */
	public void drawHorizontal(SpiderChartSwtGraphics g, Font ontheflyFont) {
		int iconWidth = 0;
		int totalWidth = 0;
		int iconHeight = 0;
		int w = 0;
		int h = 0;

		int textHeight = g.getFontHeight();
		int textWidth = computeTextWidth(g);
		int iconSeparator = 3;
		int textSeparator = 5;

		totalWidth = (textWidth + textSeparator) * items.size();
		for(Pair<DataSeries, LineStyle> pair : items) {
			if(pair.getSecond() != null) {
				iconWidth = Math.max(iconWidth, 10);
				iconHeight = Math.max(iconHeight, 10);
			}
		}
		int itemHeight = Math.max(textHeight, iconHeight);
		totalWidth += (iconWidth + iconSeparator) * items.size();

		int toCenterX = (width - totalWidth) / 2;
		int toCenterY = (height - itemHeight) / 2;
		if(toCenterY < 0) {
			toCenterY = 0;
		}
		if(toCenterX < 0) {
			toCenterX = 0;
		}
		final int legendX1 = x + toCenterX;
		final int legendY1 = y + toCenterY;
		final int legendX2 = x + toCenterX + totalWidth;
		final int legendY2 = y + toCenterY + itemHeight;
		if(getBackgroundFillStyle() != null) {
			getBackgroundFillStyle().draw(g, legendX1 - margin, legendY1 - margin,
					legendX2 + margin, legendY2 + margin);
		}
		if(getLineStyle() != null) {
			getLineStyle().drawRect(g, legendX1 - margin, legendY1 - margin, legendX2 + margin,
					legendY2 + margin);
		}
		int offset = 0;
		for(int i = 1; i <= items.size(); i++) {
			Pair<DataSeries, LineStyle> pair = items.get(i - 1);
			g.drawText(pair.getFirst().getName(), toCenterX + offset + iconWidth + iconSeparator +
					x, toCenterY + y + itemHeight, getForegroundColor(), ontheflyFont);
			offset = offset + iconWidth + iconSeparator + textWidth + textSeparator;
		}
		offset = 0;
		for(int i = 1; i <= items.size(); i++) {
			Pair<DataSeries, LineStyle> pair = items.get(i - 1);
			// if(icon instanceof AbstractChartImage) {
			// g.drawImage((AbstractChartImage)icon, toCenterX + x + offset, toCenterY +
			// y);
			// }
			if(pair.getSecond() != null) {
				LineStyle l = pair.getSecond();
				l.draw(g, toCenterX + x + offset, toCenterY + y + iconHeight / 2, toCenterX + x +
						iconWidth - 2 + offset, toCenterY + y + iconHeight / 2);
			}
			// FIXME:
			// if(icon instanceof FillStyle) {
			// int sidelentgh = iconWidth / 2;
			//
			// FillStyle f = (FillStyle)icon;
			// f.draw(g, toCenterX + x + offset, toCenterY + y, toCenterX + x +
			// offset + sidelentgh, toCenterY + y + sidelentgh);
			// }
			offset = offset + iconWidth + iconSeparator + textWidth + textSeparator;
		}
	}

	/** draws the graphics vertically */
	public void drawVertical(SpiderChartSwtGraphics g, Font ontheflyFont) {
		int iconWidth = 0;
		int iconHeight = 0;
		int h = 0;
		int textHeight = g.getFontHeight();
		int textWidth = computeTextWidth(g);

		for(Pair<DataSeries, LineStyle> pair : items) {
			if(pair.getSecond() != null) {
				iconWidth = Math.max(iconWidth, 10);
				iconHeight = Math.max(iconHeight, 10);
			}
		}
		int itemHeight = Math.max(textHeight, iconHeight);

		int toCenterX = Math.max(0, (width - (iconWidth + textWidth)) / 2);
		int toCenterY = Math.max(0, (height - items.size() * itemHeight) / 2);

		int legendX1 = x + toCenterX;
		int legendY1 = y + toCenterY;
		int legendX2 = x + toCenterX + iconWidth + textWidth;
		int legendY2 = y + toCenterY + items.size() * itemHeight;
		int x1 = legendX1 - margin;
		int y1 = legendY1 - margin;
		int x2 = legendX2 + margin;
		int y2 = legendY2 + margin;

		if(getBackgroundFillStyle() != null) {
			getBackgroundFillStyle().draw(g, x1, y1 + verticalOffset, x2, y2 + verticalOffset);
		}
		if(getLineStyle() != null) {
			getLineStyle().drawRect(g, x1, y1 + verticalOffset, x2, y2 + verticalOffset);
		}
		for(int i = 1; i <= items.size(); i++) {
			Pair<DataSeries, LineStyle> pair = items.get(i - 1);
			g.drawText(pair.getFirst().getName(), toCenterX + iconWidth + x, toCenterY + y + i *
					itemHeight + verticalOffset, getForegroundColor(), ontheflyFont);
		}
		for(int i = 1; i <= items.size(); i++) {
			// FIXME: Object
			Pair<DataSeries, LineStyle> pair = items.get(i - 1);
			if(pair.getSecond() != null) {
				LineStyle l = pair.getSecond();
				l.setWidth(10);
				l.draw(g, toCenterX + x, toCenterY + y + iconHeight / 2 + (i - 1) * itemHeight +
						verticalOffset, toCenterX + x + iconWidth - 2, toCenterY + y + iconHeight /
						2 + (i - 1) * itemHeight + verticalOffset);
			}
		}
	}

	/** */
	private int computeTextWidth(SpiderChartSwtGraphics g) {
		int textWidth = 0;
		for(Pair<DataSeries, LineStyle> pair : items) {
			int w = g.getFontWidth(pair.getFirst().getName());
			if(w > textWidth) {
				textWidth = w;
			}
		}
		return textWidth;
	}

	/** */
	public boolean isVerticalLayout() {
		return verticalLayout;
	}

	/** Sets the label. */
	public void setLabel(String label) {
		this.labelText = label;
	}

	/** Sets the margin. */
	public void setMargin(int margin) {
		this.margin = margin;
	}

	/** Sets the vertical offset. */
	public void setOffset(int offset) {
		this.verticalOffset = offset;
	}

	/** Sets the vertical layout flag. */
	public void setVerticalLayout(boolean verticalLayout) {
		this.verticalLayout = verticalLayout;
	}
}
