/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2016 fortiss GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.spiderchart.sequence;

import static org.fortiss.tooling.spiderchart.gc.Fonts.VERDANA;
import static org.fortiss.tooling.spiderchart.style.LineStyle.NORMAL_LINE;
import static org.fortiss.tooling.spiderchart.util.ChartUtil.wrapValues;

import java.util.List;
import java.util.stream.Collectors;

import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.FontData;
import org.fortiss.tooling.base.model.visualization.DataPoint;
import org.fortiss.tooling.base.model.visualization.DataSet;
import org.fortiss.tooling.spiderchart.gc.AbstractChartImage;
import org.fortiss.tooling.spiderchart.plotter.SpiderChartPlot;
import org.fortiss.tooling.spiderchart.style.FillStyle;
import org.fortiss.tooling.spiderchart.style.LineStyle;

/**
 * Represents spider chart data sequence of points for plotting
 *
 * @author mondal
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating GREEN Hash: 13207B1B51F8F60D2122999D2EDB0603
 */
public final class LineDataSequence extends DataSequence {

	/** */
	private static final int STARTING_X_VALUE = 0;

	/**
	 * Static Factory to create instance of {@link LineDataSequence}
	 */
	public static LineDataSequence of(final LineStyle style, final double... values) {
		return new LineDataSequence(values, style);
	}

	/**
	 * Static Factory to create instance of {@link LineDataSequence}
	 */
	public static <E extends Enum<E>> LineDataSequence of(SpiderChartPlot oldObjectToCopyData,
			final Object... values) {
		final double[] wrappedValues = wrapValues(values);
		final LineDataSequence seq =
				new LineDataSequence(wrappedValues, new LineStyle(2,
						oldObjectToCopyData.areaColor(), NORMAL_LINE));
		seq.valueFont = new FontData(VERDANA.getFontName(), SWT.BOLD, 12);
		seq.fillStyle = new FillStyle(oldObjectToCopyData.areaColor(), 0.5f);
		seq.drawPoint = true;
		return seq;
	}

	/**
	 * Static Factory to create instance of {@link LineDataSequence}
	 */
	public static <E extends Enum<E>> LineDataSequence of(int swtColor, final Object... values) {
		final double[] wrappedValues = wrapValues(values);
		final LineDataSequence seq =
				new LineDataSequence(wrappedValues, new LineStyle(2, swtColor, NORMAL_LINE));
		seq.valueFont = new FontData(VERDANA.getFontName(), SWT.BOLD, 12);
		seq.fillStyle = new FillStyle(swtColor, 0.5f);
		seq.drawPoint = true;
		return seq;
	}

	/**
	 * Static Factory to create instance of {@link LineDataSequence}
	 */
	public static <E extends Enum<E>> LineDataSequence of(int swtColor, final DataSet dataSet) {
		List<DataPoint<?>> datapoints = dataSet.getPoints();
		List<Object> values =
				datapoints.stream().map(datapoint -> datapoint.getStart())
						.collect(Collectors.toList());
		final double[] wrappedValues = wrapValues(values.toArray());
		final LineDataSequence seq =
				new LineDataSequence(wrappedValues, new LineStyle(2, swtColor, NORMAL_LINE));
		seq.valueFont = new FontData(VERDANA.getFontName(), SWT.BOLD, 12);
		seq.fillStyle = new FillStyle(swtColor, 0.5f);
		seq.drawPoint = true;
		return seq;
	}

	/** Draw Point Configuration */
	private boolean drawPoint = false;

	/** Fillstyle to be used */
	private FillStyle fillStyle = null;

	/** Icon to be used */
	private AbstractChartImage icon = null;

	/** Line Type */
	private int lineType = 0;

	/** Data Point Color */
	private int swtPointColor = SWT.COLOR_BLACK;

	/** Line Style */
	private LineStyle style = null;

	/** value Color */
	private int swtValueColor = SWT.COLOR_BLACK;

	/** value font */
	private FontData valueFont = null;

	/** */
	private LineStyle vstyle = null;

	/** Constructor */
	public LineDataSequence(final double[] x, final double[] y, final LineStyle s) {
		super(x, y);
		this.style = s;
	}

	/** Constructor */
	public LineDataSequence(final double[] y, final LineStyle s) {
		super(y, STARTING_X_VALUE);
		this.style = s;
	}

	/** Constructor */
	public LineDataSequence(final Double[] x, final Double[] y, final LineStyle s) {
		super(x, y);
		this.style = s;
	}

	/** Constructor */
	public LineDataSequence(final Double[] y, final LineStyle s) {
		super(y, STARTING_X_VALUE);
		this.style = s;
	}

	/** Constructor */
	public LineDataSequence(final LineStyle s) {
		this.style = s;
	}

	/**
	 * @return the fillStyle
	 */
	public FillStyle getFillStyle() {
		return this.fillStyle;
	}

	/**
	 * @return the icon
	 */
	public AbstractChartImage getIcon() {
		return this.icon;
	}

	/**
	 * @return the lineType
	 */
	public int getLineType() {
		return this.lineType;
	}

	/**
	 * @return the pointColor
	 */
	public int getPointColor() {
		return swtPointColor;
	}

	/**
	 * @return the style
	 */
	public LineStyle getStyle() {
		return this.style;
	}

	/**
	 * @return the valueColor
	 */
	public int getValueColor() {
		return swtValueColor;
	}

	/**
	 * @return the valueFont
	 */
	public FontData getValueFontData() {
		return this.valueFont;
	}

	/**
	 * @return the vstyle
	 */
	public LineStyle getVstyle() {
		return this.vstyle;
	}

	/**
	 * @return the drawPoint
	 */
	public boolean isDrawPoint() {
		return this.drawPoint;
	}

	/**
	 * @param drawPoint
	 *            the drawPoint to set
	 */
	public void setDrawPoint(final boolean drawPoint) {
		this.drawPoint = drawPoint;
	}

	/**
	 * @param fillStyle
	 *            the fillStyle to set
	 */
	public void setFillStyle(final FillStyle fillStyle) {
		this.fillStyle = fillStyle;
	}

	/**
	 * @param icon
	 *            the icon to set
	 */
	public void setIcon(final AbstractChartImage icon) {
		this.icon = icon;
	}

	/**
	 * @param lineType
	 *            the lineType to set
	 */
	public void setLineType(final int lineType) {
		this.lineType = lineType;
	}

	/**
	 * @param pointColor
	 *            the pointColor to set
	 */
	public void setPointColor(int pointColor) {
		this.swtPointColor = pointColor;
	}

	/**
	 * @param style
	 *            the style to set
	 */
	public void setStyle(final LineStyle style) {
		this.style = style;
	}

	/**
	 * @param valueColor
	 *            the valueColor to set
	 */
	public void setValueColor(int valueColor) {
		this.swtValueColor = valueColor;
	}

	/**
	 * @param valueFont
	 *            the valueFont to set
	 */
	public void setValueFontData(FontData valueFont) {
		this.valueFont = valueFont;
	}

	/**
	 * @param vstyle
	 *            the vstyle to set
	 */
	public void setVstyle(final LineStyle vstyle) {
		this.vstyle = vstyle;
	}

	/** {@inheritDoc} */
	@Override
	public void dispose() {
		if(icon != null) {
			icon.dispose();
		}
	}
}
