/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2016 fortiss GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.spiderchart.style;

import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.widgets.Display;
import org.fortiss.tooling.spiderchart.gc.AbstractChartImage;
import org.fortiss.tooling.spiderchart.gc.swt.SpiderChartSwtGraphics;

/**
 * Represents a style to be used in SWT Chart Graphics
 *
 * @author mondal
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating GREEN Hash: 13207B1B51F8F60D2122999D2EDB0603
 */
public final class FillStyle {

	/** Gradient Horizontal Setting */
	public static final int GRADIENT_HORIZONTAL = 1;

	/** Gradient Vertical Setting */
	public static final int GRADIENT_VERTICAL = 2;

	/** No Gradient Setting */
	public static final int NO_GRADIENT = 0;

	/** Needed for Gradient setting */
	private float alphaValue = 1.0F;

	/** Color instance to be used for the fill style */
	private final int swtColor;

	/** Spider Chart Canvas Background Color */
	private int colorFrom = SWT.COLOR_CYAN;

	/** Gradient Color for the shadow */
	private int colorUntil = SWT.COLOR_WHITE;

	/** */
	private Object composite = null;

	/** */
	private int gradientType = NO_GRADIENT;

	/** Image if needed for the texture */
	private AbstractChartImage textureImage = null;

	/** Constructor */
	public FillStyle(int swtColor) {
		this.swtColor = swtColor;
	}

	/** Constructor */
	public FillStyle(int swtColor, final float f) {
		this.swtColor = swtColor;
		this.alphaValue = f;
	}

	/** Constructor */
	public FillStyle(final AbstractChartImage i) {
		this.textureImage = i;
		this.swtColor = SWT.COLOR_WHITE;
	}

	/** Returns the current SWT system color. */
	private Color currentColor(int swtColor) {
		return Display.getCurrent().getSystemColor(swtColor);
	}

	/** Draws the style */
	public void draw(SpiderChartSwtGraphics g, int x1, int y1, int x2, int y2) {
		if(x1 > x2) {
			final int xtmp = x2;
			x2 = x1;
			x1 = xtmp;
		}
		if(y1 > y2) {
			final int ytmp = y2;
			y2 = y1;
			y1 = ytmp;
		}
		g.setTexture(this.textureImage);
		if(this.gradientType != NO_GRADIENT) {
			g.createFadeArea(currentColor(colorFrom), currentColor(colorUntil), x1, y1, x2 - x1,
					y2 - y1, this.gradientType == GRADIENT_VERTICAL);
		} else {
			this.setAlpha(g);
			g.fillRect(x1, y1, x2 - x1, y2 - y1, currentColor(swtColor));
			this.resetAlpha(g);
		}
	}

	/** Draws the fill style */
	public void draw(SpiderChartSwtGraphics g, Color backgroundCanvasColor, int x1, int y1, int x2,
			int y2) {
		if(x1 > x2) {
			final int xtmp = x2;
			x2 = x1;
			x1 = xtmp;
		}
		if(y1 > y2) {
			final int ytmp = y2;
			y2 = y1;
			y1 = ytmp;
		}
		g.setTexture(this.textureImage);
		if(this.gradientType != NO_GRADIENT) {
			g.createFadeArea(backgroundCanvasColor, currentColor(colorUntil), x1, y1, x2 - x1, y2 -
					y1, this.gradientType == GRADIENT_VERTICAL);
		} else {
			this.setAlpha(g);
			g.fillRect(x1, y1, x2 - x1, y2 - y1, currentColor(swtColor));
			this.resetAlpha(g);
		}
	}

	/** Draws arc */
	public void drawArc(SpiderChartSwtGraphics g, int x, int y, int w, int h, int a1, int a2) {
		g.setTexture(this.textureImage);
		this.setAlpha(g);
		g.fillArc(x, y, w, h, a1, a2, currentColor(swtColor));
		this.resetAlpha(g);
	}

	/** Draws Polygon */
	public void drawPolygon(SpiderChartSwtGraphics g, int[] x1, int[] y1, int num) {
		g.setTexture(this.textureImage);
		this.setAlpha(g);
		g.fillPolygon(x1, y1, num, currentColor(swtColor));
		this.resetAlpha(g);
	}

	/** Draws Round Rectangle */
	public void drawRoundRect(SpiderChartSwtGraphics g, int x1, int y1, int x2, int y2) {
		g.setTexture(this.textureImage);
		this.setAlpha(g);
		g.fillRoundRect(x1, y1, x2 - x1, y2 - y1, currentColor(swtColor));
		this.resetAlpha(g);
	}

	/**
	 * @return the gradientType
	 */
	public int getGradientType() {
		return this.gradientType;
	}

	/** Resets alpha for the composite */
	private void resetAlpha(SpiderChartSwtGraphics g) {
		if(this.composite != null) {
			g.setAlphaComposite(this.composite);
		}
		this.composite = null;
	}

	/** Setter for alpha value */
	private void setAlpha(SpiderChartSwtGraphics g) {
		if(this.alphaValue != 1.0F) {
			this.composite = g.getAlphaComposite();
			g.setAlpha(this.alphaValue);
		}
	}

	/**
	 * @param gradientType
	 *            the gradientType to set
	 */
	public void setGradientType(final int gradientType) {
		this.gradientType = gradientType;
	}

	/** {@inheritDoc} */
	@Override
	public String toString() {
		if(this.gradientType != NO_GRADIENT) {
			return currentColor(colorFrom).toString() + ";" + currentColor(colorUntil).toString();
		}
		if(this.alphaValue != 1.0F) {
			return currentColor(swtColor).toString() + "|" + this.alphaValue;
		}
		return currentColor(swtColor).toString();
	}
}
