/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2016 fortiss GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.spiderchart.swt;

import static org.eclipse.swt.SWT.CURSOR_ARROW;
import static org.eclipse.swt.SWT.CURSOR_HAND;
import static org.eclipse.swt.SWT.NONE;

import org.eclipse.swt.events.ControlEvent;
import org.eclipse.swt.events.ControlListener;
import org.eclipse.swt.graphics.Cursor;
import org.eclipse.swt.widgets.Composite;
import org.fortiss.tooling.spiderchart.SpiderChartOld;
import org.fortiss.tooling.spiderchart.listener.SpiderChartAdapter;

/**
 * Represents a viewer on the canvas to display the chart
 *
 * @author mondal
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating GREEN Hash: 13207B1B51F8F60D2122999D2EDB0603
 */
public final class SpiderChartViewer extends Composite {

	/** the actual canvas to be used */
	private SpiderChartCanvas canvas = null;

	/** allows to change pointer on hovering data points */
	private boolean changePointer = true;

	/** Default Chart Listeners for hovering behaviors */
	private final SpiderChartAdapter chartAdapter = new SpiderChartAdapter() {
		/** {@inheritDoc} */
		@Override
		public void onChartEvent(final SpiderChartOld c, final int type) {
			if(type == 2) {
				SpiderChartViewer.this.canvas.setCursor(SpiderChartViewer.this.pointCursor);
			}
			if(type == 3) {
				SpiderChartViewer.this.canvas.setCursor(SpiderChartViewer.this.defaultCursor);
			}
		}
	};

	/** Active Zoom Percentage */
	private int currentZoom = 100;

	/** Default cursor object */
	private Cursor defaultCursor = null;

	/** height of the canvas (used to restore from zoom) */
	private int lastHeight = 0;

	/** width of the canvas (used to restore from zoom) */
	private int lastWidth = 0;

	/** Actual height of the canvas */
	private int originalHeight = -1;

	/** Actual width of the canvas */
	private int originalWidth = -1;

	/** Cursor object in use */
	private Cursor pointCursor = null;

	/** Constructor */
	public SpiderChartViewer(Composite parent, final int style) {
		super(parent, style);

		canvas = new SpiderChartCanvas(this, NONE);
		canvas.setFocus();

		this.addControlListener(new ControlListener() {
			/** {@inheritDoc} */
			@Override
			public void controlMoved(ControlEvent e) {
				// Not used
			}

			/** {@inheritDoc} */
			@Override
			public void controlResized(ControlEvent e) {
				if(SpiderChartViewer.this.canvas.getChart() == null) {
					return;
				}
				setCanvasSize();
			}
		});
		this.defaultCursor = new Cursor(parent.getDisplay(), CURSOR_ARROW);
		this.pointCursor = new Cursor(parent.getDisplay(), CURSOR_HAND);
	}

	/** {@inheritDoc} */
	@Override
	public void dispose() {
		super.dispose();
		if(this.pointCursor != null && !this.pointCursor.isDisposed()) {
			this.pointCursor.dispose();
		}
		if(this.defaultCursor != null && !this.defaultCursor.isDisposed()) {
			this.defaultCursor.dispose();
		}
	}

	/** getter for the canvas in use */
	public SpiderChartCanvas getCanvas() {
		return this.canvas;
	}

	/** getter for the chart in use */
	public SpiderChartOld getChart() {
		return this.canvas.getChart();
	}

	/** getter for the chart listener in use */
	public SpiderChartAdapter getChartAdapter() {
		return this.chartAdapter;
	}

	/** getter for the current zoom percentage */
	public int getCurrentZoom() {
		return this.currentZoom;
	}

	/** getter for the default cursor */
	public Cursor getDefaultCursor() {
		return this.defaultCursor;
	}

	/** getter for height after zoom */
	public int getLastHeight() {
		return this.lastHeight;
	}

	/** getter for width after zoom */
	public int getLastWidth() {
		return this.lastWidth;
	}

	/** getter for original canvas height */
	public int getOriginalHeight() {
		return this.originalHeight;
	}

	/** getter for original canvas width */
	public int getOriginalWidth() {
		return this.originalWidth;
	}

	/** getter for cursor in use */
	public Cursor getPointCursor() {
		return this.pointCursor;
	}

	/**
	 * getter for configuration of changing pointer on data point mouse hover
	 */
	public boolean isChangePointer() {
		return this.changePointer;
	}

	/** Redraws the chart */
	public void redrawChart() {
		this.canvas.redraw();
	}

	/** Resets the chart */
	private void resetChart() {
		this.lastWidth = 0;
		this.lastHeight = 0;
	}

	/** setter for chart canvas */
	public void setCanvas(final SpiderChartCanvas canvas) {
		this.canvas = canvas;
	}

	/** setter for changing pointer */
	public void setChangePointer(final boolean changePointer) {
		this.changePointer = changePointer;
	}

	/** setter for chart instance */
	public void setChart(final SpiderChartOld c) {
		if(this.canvas.getChart() != null) {
			this.canvas.getChart().removeChartListener(this.chartAdapter);
		}
		this.canvas.setChart(c);

		this.originalHeight = this.canvas.getChart().getVirtualHeight();
		this.originalWidth = this.canvas.getChart().getVirtualWidth();

		this.resetChart();
		setCanvasSize();
		if(this.changePointer) {
			this.canvas.getChart().addChartListener(this.chartAdapter);
		}
	}

	/**
	 * 
	 */
	private void setCanvasSize() {
		this.canvas.setSize(this.getSize().x, this.getSize().y);
		this.canvas.setLocation(0, 0);
	}

	/** setter for current zoom percentage */
	public void setCurrentZoom(final int currentZoom) {
		this.currentZoom = currentZoom;
	}

	/** setter for default cursor */
	public void setDefaultCursor(final Cursor defaultCursor) {
		this.defaultCursor = defaultCursor;
	}

	/** setter for last height after zoom */
	public void setLastHeight(final int lastHeight) {
		this.lastHeight = lastHeight;
	}

	/** setter for last width after zoom */
	public void setLastWidth(final int lastWidth) {
		this.lastWidth = lastWidth;
	}

	/** setter for original height of the canvas */
	public void setOriginalHeight(final int originalHeight) {
		this.originalHeight = originalHeight;
	}

	/** setter for original width of the canvas */
	public void setOriginalWidth(final int originalWidth) {
		this.originalWidth = originalWidth;
	}

	/** setter for the cursor instance in use */
	public void setPointCursor(final Cursor pointCursor) {
		this.pointCursor = pointCursor;
	}
}
